
# q-expansion of newform 5265.2.a.i, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5265
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [-4], [0], [1], [-3], [2], [3], [6], [-4], [8], [-12], [-1], [0], [-3], [6], [11], [-4], [6], [8], [-7], [6], [-18], [14], [-15], [-4], [-3], [-10], [3], [-16], [-21], [6], [5], [18], [-22], [-13], [-16], [12], [15], [9], [-25], [3], [-4], [12], [-7], [5], [-22], [-12], [2], [27], [-18], [-10], [-27], [3], [-3], [6], [20], [-22], [-12], [-13], [0], [-4], [-24], [-10], [-30], [20], [5], [15], [-10], [-24], [18], [17], [35], [-16], [-24], [-9], [-4], [18], [32], [15], [2], [24], [-37], [-13], [9], [0], [8], [0], [26], [-3], [6], [38], [-36], [-4], [-33], [-30], [6], [-28], [20], [-40], [6], [-39], [42], [-28], [-28], [18], [30], [-33], [29], [-7], [26], [6], [-10], [-16], [-42], [-22], [9], [30], [0], [-22], [-7], [42], [-36], [44], [27], [2], [24], [-31], [-22], [-34], [42], [-16], [-19], [12], [32], [-18], [-46], [54], [9], [-4], [-30], [41], [18], [14], [-48], [-10], [54], [-37], [-6], [-10], [-15], [8], [27], [-1], [39], [11], [-54], [-31], [18], [-12], [6], [-28], [-12], [24], [-24], [-55], [53], [-16], [42], [54], [-4], [-27], [-10], [5], [30], [-10], [-6], [56], [17], [-28], [-45], [-19], [6], [36], [-9], [-34], [44], [38], [-36], [59], [-18], [-1], [18], [24], [54], [2], [-31], [-12], [-24], [-36], [32], [20], [50], [-24], [-3], [8], [12], [30], [23], [11], [-9], [8], [54], [66], [-34], [47], [-66], [-42], [6], [-73], [8], [-30], [-52], [0], [-19], [33], [57], [-64], [30], [-31], [-25], [-46], [-45], [-61], [-24], [-34], [66], [-21], [15], [12], [-25], [11], [20], [72], [-57], [-22], [66], [26], [63], [-22], [30], [-72], [71], [39], [-30], [-19], [-10], [-51], [-70], [-37], [27], [20], [-10], [-24], [-19], [-48], [-30], [-22], [54], [-43], [38], [-46], [-4], [14], [62], [18], [-4], [50], [-60], [-63], [-4], [-63], [80], [-4], [63], [-34], [48], [32], [33], [-57], [3], [-30], [-12], [-25], [-33], [-73], [30], [33], [-16], [-61], [-36], [-31], [-48], [35], [-34], [33], [-70], [-48], [-7], [-21], [-66], [30], [20], [18], [17], [78], [-48], [38], [-57], [35], [68], [-57], [38], [72], [-25], [80], [26], [0], [-9], [26], [-81], [-37], [78], [26], [48], [-4], [18], [-46], [29], [26], [-9], [66], [-64], [-36], [-21], [-55], [38], [6], [-75], [14], [38], [66], [-7], [-67], [3], [-54], [-72], [-33], [96], [-10], [-69], [-51], [-66], [-4], [14], [42], [-40], [11], [0], [29], [66], [-57], [-13], [-43], [-60], [12], [56], [54], [17], [-12], [36], [-64], [-60], [-70], [-60], [-82], [-58], [8], [48], [-76], [0], [48], [-73], [12], [53], [56], [75], [-55], [-42], [-40], [90], [-46], [-90], [-24], [-16], [-34], [24], [-58], [0], [-34], [-87], [-12], [17], [-13], [-57], [54], [-25], [66], [-36], [21], [20], [48], [-84], [-16], [-60], [99], [24], [68], [69]]
